@@ -17,7 +17,7 @@ Usage:
 
     logger.debug("hello")
     logger.info("info")
-    logger.warn("warn")
+    logger.warning("warn")
     logger.error("error")
 
 In order to also log to a file, just use `logzero.logfile(..)`:
@@ -39,6 +39,8 @@ import os
 import sys
 import logging
 from logzero.colors import Fore as ForegroundColors
+from logzero.jsonlogger import JsonFormatter
+
 from logging.handlers import RotatingFileHandler, SysLogHandler
 
 try:
@@ -48,7 +50,7 @@ except ImportError:
 
 __author__ = """Chris Hager"""
 __email__ = 'chris@linuxuser.at'
-__version__ = '1.5.0'
+__version__ = '1.6.2'
 
 # Python 2+3 compatibility settings for logger
 bytes_type = bytes
@@ -61,6 +63,17 @@ else:
     unicode_type = unicode  # noqa
     basestring_type = basestring  # noqa
 
+# Formatter defaults
+DEFAULT_FORMAT = '%(color)s[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s'
+DEFAULT_DATE_FORMAT = '%y%m%d %H:%M:%S'
+DEFAULT_COLORS = {
+    logging.DEBUG: ForegroundColors.CYAN,
+    logging.INFO: ForegroundColors.GREEN,
+    logging.WARNING: ForegroundColors.YELLOW,
+    logging.ERROR: ForegroundColors.RED,
+    logging.CRITICAL: ForegroundColors.RED
+}
+
 # Name of the internal default logger
 LOGZERO_DEFAULT_LOGGER = "logzero_default"
 
@@ -84,7 +97,7 @@ if os.name == 'nt':
     colorama_init()
 
 
-def setup_logger(name=None, logfile=None, level=logging.DEBUG, formatter=None, maxBytes=0, backupCount=0, fileLoglevel=None, disableStderrLogger=False):
+def setup_logger(name=__name__, logfile=None, level=logging.DEBUG, formatter=None, maxBytes=0, backupCount=0, fileLoglevel=None, disableStderrLogger=False, isRootLogger=False, json=False):
     """
     Configures and returns a fully configured logger instance, no hassles.
     If a logger with the specified name already exists, it returns the existing instance,
@@ -109,11 +122,19 @@ def setup_logger(name=None, logfile=None, level=logging.DEBUG, formatter=None, m
     :arg int backupCount: Number of backups to keep. Defaults to 0, rollover never occurs.
     :arg int fileLoglevel: Minimum `logging-level <https://docs.python.org/2/library/logging.html#logging-levels>`_ for the file logger (is not set, it will use the loglevel from the ``level`` argument)
     :arg bool disableStderrLogger: Should the default stderr logger be disabled. Defaults to False.
+    :arg bool isRootLogger: If True then returns a root logger. Defaults to False. (see also the `Python docs <https://docs.python.org/3/library/logging.html#logging.getLogger>`_).
+    :arg bool json: If True then log in JSON format. Defaults to False. (uses `python-json-logger <https://github.com/madzak/python-json-logger>`_).
     :return: A fully configured Python logging `Logger object <https://docs.python.org/2/library/logging.html#logger-objects>`_ you can use with ``.debug("msg")``, etc.
     """
-    _logger = logging.getLogger(name or __name__)
+    _logger = logging.getLogger(None if isRootLogger else name)
     _logger.propagate = False
-    _logger.setLevel(level)
+
+    # set the minimum level needed for the logger itself (the lowest handler level)
+    minLevel = fileLoglevel if fileLoglevel and fileLoglevel < level else level
+    _logger.setLevel(minLevel)
+
+    # Setup default formatter
+    _formatter = _get_json_formatter() if json else formatter or LogFormatter()
 
     # Reconfigure existing handlers
     stderr_stream_handler = None
@@ -129,7 +150,7 @@ def setup_logger(name=None, logfile=None, level=logging.DEBUG, formatter=None, m
 
         # reconfigure handler
         handler.setLevel(level)
-        handler.setFormatter(formatter or LogFormatter())
+        handler.setFormatter(_formatter)
 
     # remove the stderr handler (stream_handler) if disabled
     if disableStderrLogger:
@@ -139,14 +160,14 @@ def setup_logger(name=None, logfile=None, level=logging.DEBUG, formatter=None, m
         stderr_stream_handler = logging.StreamHandler()
         setattr(stderr_stream_handler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
         stderr_stream_handler.setLevel(level)
-        stderr_stream_handler.setFormatter(formatter or LogFormatter())
+        stderr_stream_handler.setFormatter(_formatter)
         _logger.addHandler(stderr_stream_handler)
 
     if logfile:
         rotating_filehandler = RotatingFileHandler(filename=logfile, maxBytes=maxBytes, backupCount=backupCount)
         setattr(rotating_filehandler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
         rotating_filehandler.setLevel(fileLoglevel or level)
-        rotating_filehandler.setFormatter(formatter or LogFormatter(color=False))
+        rotating_filehandler.setFormatter(_formatter)
         _logger.addHandler(rotating_filehandler)
 
     return _logger
@@ -159,15 +180,6 @@ class LogFormatter(logging.Formatter):
     * Timestamps on every log line.
     * Robust against str/bytes encoding problems.
     """
-    DEFAULT_FORMAT = '%(color)s[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s'
-    DEFAULT_DATE_FORMAT = '%y%m%d %H:%M:%S'
-    DEFAULT_COLORS = {
-        logging.DEBUG: ForegroundColors.CYAN,
-        logging.INFO: ForegroundColors.GREEN,
-        logging.WARNING: ForegroundColors.YELLOW,
-        logging.ERROR: ForegroundColors.RED
-    }
-
     def __init__(self,
                  color=True,
                  fmt=DEFAULT_FORMAT,
@@ -328,6 +340,13 @@ def reset_default_logger():
     _loglevel = logging.DEBUG
     _logfile = None
     _formatter = None
+
+    # Remove all handlers on exiting logger
+    if logger:
+        for handler in list(logger.handlers):
+            logger.removeHandler(handler)
+
+    # Resetup
     logger = setup_logger(name=LOGZERO_DEFAULT_LOGGER, logfile=_logfile, level=_loglevel, formatter=_formatter)
 
 
@@ -337,7 +356,7 @@ reset_default_logger()
 
 def loglevel(level=logging.DEBUG, update_custom_handlers=False):
     """
-    Set the minimum loglevel for the default logger (`logzero.logger`).
+    Set the minimum loglevel for the default logger (`logzero.logger`) and all handlers.
 
     This reconfigures only the internal handlers of the default logger (eg. stream and logfile).
     You can also update the loglevel for custom handlers by using `update_custom_handlers=True`.
@@ -408,22 +427,30 @@ def logfile(filename, formatter=None, mode='a', maxBytes=0, backupCount=0, encod
     :arg int loglevel: Set a custom loglevel for the file logger, else uses the current global loglevel.
     :arg bool disableStderrLogger: Should the default stderr logger be disabled. Defaults to False.
     """
-    # Step 1: If an internal RotatingFileHandler already exists, remove it
+    # First, remove any existing file logger
     __remove_internal_loggers(logger, disableStderrLogger)
 
-    # Step 2: If wanted, add the RotatingFileHandler now
-    if filename:
-        rotating_filehandler = RotatingFileHandler(filename, mode=mode, maxBytes=maxBytes, backupCount=backupCount, encoding=encoding)
+    # If no filename supplied, all is done
+    if not filename:
+        return
 
-        # Set internal attributes on this handler
-        setattr(rotating_filehandler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
-        if loglevel:
-            setattr(rotating_filehandler, LOGZERO_INTERNAL_HANDLER_IS_CUSTOM_LOGLEVEL, True)
+    # Now add
+    rotating_filehandler = RotatingFileHandler(filename, mode=mode, maxBytes=maxBytes, backupCount=backupCount, encoding=encoding)
 
-        # Configure the handler and add it to the logger
-        rotating_filehandler.setLevel(loglevel or _loglevel)
-        rotating_filehandler.setFormatter(formatter or _formatter or LogFormatter(color=False))
-        logger.addHandler(rotating_filehandler)
+    # Set internal attributes on this handler
+    setattr(rotating_filehandler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
+    if loglevel:
+        setattr(rotating_filehandler, LOGZERO_INTERNAL_HANDLER_IS_CUSTOM_LOGLEVEL, True)
+
+    # Configure the handler and add it to the logger
+    rotating_filehandler.setLevel(loglevel or _loglevel)
+    rotating_filehandler.setFormatter(formatter or _formatter or LogFormatter(color=False))
+    logger.addHandler(rotating_filehandler)
+
+    # If wanting to use a lower loglevel for the file handler, we need to reconfigure the logger level
+    # (note: this won't change the StreamHandler loglevel)
+    if loglevel and loglevel < logger.level:
+        logger.setLevel(loglevel)
 
 
 def __remove_internal_loggers(logger_to_update, disableStderrLogger=True):
@@ -460,6 +487,37 @@ def syslog(logger_to_update=logger, facility=SysLogHandler.LOG_USER, disableStde
     return syslog_handler
 
 
+def json(enable=True, update_custom_handlers=False):
+    """
+    Enable/disable json logging for all handlers.
+    """
+
+    formatter(_get_json_formatter() if enable else LogFormatter(), update_custom_handlers=update_custom_handlers)
+
+
+def _get_json_formatter():
+    supported_keys = [
+        'asctime',
+        'filename',
+        'funcName',
+        'levelname',
+        'levelno',
+        'lineno',
+        'module',
+        'message',
+        'name',
+        'pathname',
+        'process',
+        'processName',
+        'threadName'
+    ]
+
+    def log_format(x):
+        return ['%({0:s})s'.format(i) for i in x]
+    custom_format = ' '.join(log_format(supported_keys))
+    return JsonFormatter(custom_format)
+
+
 def log_function_call(func):
     @functools.wraps(func)
     def wrap(*args, **kwargs):
